@@ -1,6 +1,9 @@
 #! /usr/bin/env python
 
-import sys, os, tempfile, psycopg2
+import sys
+import os
+import tempfile
+import psycopg2
 
 if len(sys.argv) != 3:
     print('usage: mkauth DSTFN CONNSTR')
@@ -20,7 +23,8 @@ curs.execute("select usename, passwd from pg_shadow order by 1")
 lines = []
 for user, psw in curs.fetchall():
     user = user.replace('"', '""')
-    if not psw: psw = ''
+    if not psw:
+        psw = ''
     psw = psw.replace('"', '""')
     lines.append('"%s" "%s" ""\n' % (user, psw))
 db.commit()
@@ -28,7 +32,7 @@ cur = "".join(lines)
 
 # if changed, replace data securely
 if old != cur:
-    fd, tmpfn = tempfile.mkstemp(dir = os.path.split(fn)[0])
+    fd, tmpfn = tempfile.mkstemp(dir=os.path.split(fn)[0])
     f = os.fdopen(fd, 'w')
     f.write(cur)
     f.close()
