@@ -45,7 +45,6 @@
 #define SD_LISTEN_FDS_START 3
 #define sd_is_socket(fd, f, t, l) (0)
 #define sd_listen_fds(ue) (0)
-#define sd_listen_fds_with_names(ue, n) (0)
 #define sd_notify(ue, s)
 #define sd_notifyf(ue, f, ...)
 #endif
@@ -315,6 +314,7 @@ struct PgUser {
 	uint8_t scram_ClientKey[32];
 	uint8_t scram_ServerKey[32];
 	bool has_scram_keys;		/* true if the above two are valid */
+	bool mock_auth;			/* not a real user, only for mock auth */
 	int pool_mode;
 	int max_user_connections;	/* how much server connections are allowed */
 	int connection_count;	/* how much connections are used by user now */
@@ -371,7 +371,7 @@ struct PgSocket {
 	PgSocket *link;		/* the dest of packets */
 	PgPool *pool;		/* parent pool, if NULL not yet assigned */
 
-	PgUser *auth_user;	/* presented login, for client it may differ from pool->user */
+	PgUser *login_user;	/* presented login, for client it may differ from pool->user */
 
 	int client_auth_type;	/* auth method decided by hba */
 
@@ -583,5 +583,5 @@ void load_config(void);
 
 
 bool set_config_param(const char *key, const char *val);
-void config_for_each(void (*param_cb)(void *arg, const char *name, const char *val, bool reloadable),
+void config_for_each(void (*param_cb)(void *arg, const char *name, const char *val, const char *defval, bool reloadable),
 		     void *arg);
