@@ -41,18 +41,6 @@
 #include <sys/uio.h>
 #endif
 
-#ifndef UNIX_PATH_MAX
-#define UNIX_PATH_MAX  128 /* actual sizeof() will be applied later anyway */
-#endif
-
-/*
- * PostgreSQL type OIDs for resultsets.
- */
-
-#define INT8OID 20
-#define INT4OID 23
-#define TEXTOID 25
-
 /*
  * libc compat functions.
  */
