@@ -74,15 +74,6 @@ typedef enum { true=1, false=0 } bool;
 #define DLLIMPORT
 #endif
 
-#ifndef PRIdZ
-#define PRIdZ	"zd"		/**< printf 'd' format specifier for ssize_t */
-#define PRIiZ	"zi"		/**< printf 'i' format specifier for ssize_t */
-#define PRIoZ	"zo"		/**< printf 'o' format specifier for size_t */
-#define PRIuZ	"zu"		/**< printf 'u' format specifier for size_t */
-#define PRIxZ	"zx"		/**< printf 'x' format specifier for size_t */
-#define PRIXZ	"zX"		/**< printf 'X' format specifier for size_t */
-#endif
-
 /** give offset of a field inside struct */
 #ifndef offsetof
 #define offsetof(type, field) ((unsigned long)&(((type *)0)->field))
@@ -241,7 +232,11 @@ typedef enum { true=1, false=0 } bool;
 
 /** Check printf-style format and arg sanity */
 #if _COMPILER_GNUC(4,0) || __has_attribute(printf)
+#ifdef __MINGW32__
+#define _PRINTF(fmtpos, argpos) __attribute__((format(__MINGW_PRINTF_FORMAT, fmtpos, argpos)))
+#else
 #define _PRINTF(fmtpos, argpos) __attribute__((format(printf, fmtpos, argpos)))
+#endif
 #else
 #define _PRINTF(fmtpos, argpos)
 #endif
