@@ -189,7 +189,7 @@ bool parse_database(void *base, const char *name, const char *connstr)
 	char *port = "5432";
 	char *username = NULL;
 	char *password = "";
-	char *auth_username = cf_auth_user;
+	char *auth_username = NULL;
 	char *client_encoding = NULL;
 	char *datestyle = NULL;
 	char *timezone = NULL;
@@ -549,12 +549,12 @@ bool load_auth_file(const char *fn)
 
 	/* No file to load? */
 	if (fn == NULL)
-		return NULL;
+		return false;
 
 	buf = load_file(fn, NULL);
 	if (buf == NULL) {
 		log_error("could not open auth_file %s: %s", fn, strerror(errno));
-		return NULL;
+		return false;
 	}
 
 	log_debug("loading auth_file: \"%s\"", fn);
