@@ -230,7 +230,7 @@ void change_server_state(PgSocket *server, SocketState newstate)
 		statlist_remove(&pool->active_server_list, &server->head);
 		break;
 	default:
-		fatal("change_server_state: bad old server state: %d", server->state);
+		fatal("bad old server state: %d", server->state);
 	}
 
 	server->state = newstate;
@@ -267,7 +267,7 @@ void change_server_state(PgSocket *server, SocketState newstate)
 		statlist_append(&pool->active_server_list, &server->head);
 		break;
 	default:
-		fatal("bad server state");
+		fatal("bad server state: %d", server->state);
 	}
 }
 
@@ -596,6 +596,10 @@ bool check_fast_fail(PgSocket *client)
 	int cnt;
 	PgPool *pool = client->pool;
 
+	/* Could be mock authentication, proceed normally */
+	if (!pool)
+		return true;
+
 	/* If last login succeeded, client can go ahead. */
 	if (!pool->last_login_failed)
 		return true;
@@ -778,7 +782,7 @@ bool release_server(PgSocket *server)
 		pool->last_connect_failed = 0;
 		break;
 	default:
-		fatal("bad server state in release_server (%d)", server->state);
+		fatal("bad server state: %d", server->state);
 	}
 
 	/* enforce lifetime immediately on release */
@@ -863,7 +867,7 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 		}
 		break;
 	default:
-		fatal("disconnect_server: bad server state (%d)", server->state);
+		fatal("bad server state: %d", server->state);
 	}
 
 	Assert(server->link == NULL);
@@ -892,16 +896,19 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 /* drop client connection */
 void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 {
-	char buf[128];
-	va_list ap;
 	usec_t now = get_cached_time();
 
-	va_start(ap, reason);
-	vsnprintf(buf, sizeof(buf), reason, ap);
-	va_end(ap);
-	reason = buf;
+	if (reason) {
+		char buf[128];
+		va_list ap;
 
-	if (cf_log_disconnections)
+		va_start(ap, reason);
+		vsnprintf(buf, sizeof(buf), reason, ap);
+		va_end(ap);
+		reason = buf;
+	}
+
+	if (cf_log_disconnections && reason)
 		slog_info(client, "closing because: %s (age=%" PRIu64 "s)", reason,
 			  (now - client->connect_time) / USEC);
 
@@ -925,7 +932,7 @@ void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 	case CL_CANCEL:
 		break;
 	default:
-		fatal("bad client state in disconnect_client: %d", client->state);
+		fatal("bad client state: %d", client->state);
 	}
 
 	/* send reason to client */
@@ -938,6 +945,10 @@ void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 	}
 
 	free_scram_state(&client->scram_state);
+	if (client->login_user && client->login_user->mock_auth) {
+		free(client->login_user);
+		client->login_user = NULL;
+	}
 
 	change_client_state(client, CL_JUSTFREE);
 	if (!sbuf_close(&client->sbuf))
@@ -1206,7 +1217,7 @@ allow_new:
 
 	/* initialize it */
 	server->pool = pool;
-	server->auth_user = server->pool->user;
+	server->login_user = server->pool->user;
 	server->connect_time = get_cached_time();
 	pool->last_connect_time = get_cached_time();
 	change_server_state(server, SV_LOGIN);
@@ -1259,7 +1270,7 @@ bool finish_client_login(PgSocket *client)
 	case CL_ACTIVE:
 		break;
 	default:
-		fatal("bad client state");
+		fatal("bad client state: %d", client->state);
 	}
 
 	client->wait_for_auth = 0;
@@ -1488,7 +1499,7 @@ bool use_server_socket(int fd, PgAddr *addr,
 
 	server->suspended = 1;
 	server->pool = pool;
-	server->auth_user = user;
+	server->login_user = user;
 	server->connect_time = server->request_time = get_cached_time();
 	server->query_start = 0;
 
