@@ -127,18 +127,18 @@ static bool add_listen(int af, const struct sockaddr *sa, int salen)
 	 * unportable, so perhaps better to avoid it.)
 	 */
 	if (af != AF_UNIX && cf_so_reuseport) {
-#if defined(SO_REUSEPORT)
-		int val = 1;
-		errpos = "setsockopt/SO_REUSEPORT";
-		res = setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &val, sizeof(val));
-		if (res < 0)
-			goto failed;
-#elif defined(SO_REUSEPORT_LB)
+#if defined(SO_REUSEPORT_LB)
 		int val = 1;
 		errpos = "setsockopt/SO_REUSEPORT_LB";
 		res = setsockopt(sock, SOL_SOCKET, SO_REUSEPORT_LB, &val, sizeof(val));
 		if (res < 0)
 			goto failed;
+#elif defined(SO_REUSEPORT)
+		int val = 1;
+		errpos = "setsockopt/SO_REUSEPORT";
+		res = setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &val, sizeof(val));
+		if (res < 0)
+			goto failed;
 #else
 		die("so_reuseport not supported on this platform");
 #endif
@@ -469,9 +469,8 @@ static bool parse_addr(void *arg, const char *addr)
 void pooler_setup(void)
 {
 	int n;
-	char **names = NULL;
 
-	n = sd_listen_fds_with_names(0, &names);
+	n = sd_listen_fds(0);
 	if (n > 0) {
 		if (cf_listen_addr && *cf_listen_addr)
 			log_warning("sockets passed from service manager, cf_listen_addr ignored");
@@ -504,8 +503,8 @@ void pooler_setup(void)
 				tune_accept(fd, cf_tcp_defer_accept);
 			}
 			if (!ok)
-				die("failed to set up socket passed from service manager (%s, fd %d)", names[i], fd);
-			log_info("socket passed from service manager (%s, fd %d)", names[i], fd);
+				die("failed to set up socket passed from service manager (fd %d)", fd);
+			log_info("socket passed from service manager (fd %d)", fd);
 			statlist_append(&sock_list, &ls->node);
 		}
 	} else {
