@@ -74,13 +74,13 @@ static void sbuf_try_resync(SBuf *sbuf, bool release);
 static bool sbuf_wait_for_data(SBuf *sbuf) _MUSTCHECK;
 static void sbuf_main_loop(SBuf *sbuf, bool skip_recv);
 static bool sbuf_call_proto(SBuf *sbuf, int event) /* _MUSTCHECK */;
-static bool sbuf_actual_recv(SBuf *sbuf, unsigned len)  _MUSTCHECK;
+static bool sbuf_actual_recv(SBuf *sbuf, size_t len)  _MUSTCHECK;
 static bool sbuf_after_connect_check(SBuf *sbuf)  _MUSTCHECK;
 static bool handle_tls_handshake(SBuf *sbuf) /* _MUSTCHECK */;
 
 /* regular I/O */
-static int raw_sbufio_recv(struct SBuf *sbuf, void *dst, unsigned int len);
-static int raw_sbufio_send(struct SBuf *sbuf, const void *data, unsigned int len);
+static ssize_t raw_sbufio_recv(struct SBuf *sbuf, void *dst, size_t len);
+static ssize_t raw_sbufio_send(struct SBuf *sbuf, const void *data, size_t len);
 static int raw_sbufio_close(struct SBuf *sbuf);
 static const SBufIO raw_sbufio_ops = {
 	raw_sbufio_recv,
@@ -90,8 +90,8 @@ static const SBufIO raw_sbufio_ops = {
 
 /* I/O over TLS */
 #ifdef USE_TLS
-static int tls_sbufio_recv(struct SBuf *sbuf, void *dst, unsigned int len);
-static int tls_sbufio_send(struct SBuf *sbuf, const void *data, unsigned int len);
+static ssize_t tls_sbufio_recv(struct SBuf *sbuf, void *dst, size_t len);
+static ssize_t tls_sbufio_send(struct SBuf *sbuf, const void *data, size_t len);
 static int tls_sbufio_close(struct SBuf *sbuf);
 static const SBufIO tls_sbufio_ops = {
 	tls_sbufio_recv,
@@ -143,7 +143,7 @@ failed:
 }
 
 /* need to connect() to get a socket */
-bool sbuf_connect(SBuf *sbuf, const struct sockaddr *sa, int sa_len, int timeout_sec)
+bool sbuf_connect(SBuf *sbuf, const struct sockaddr *sa, socklen_t sa_len, time_t timeout_sec)
 {
 	int res, sock;
 	struct timeval timeout;
@@ -505,7 +505,8 @@ static bool sbuf_queue_send(SBuf *sbuf)
  */
 static bool sbuf_send_pending(SBuf *sbuf)
 {
-	int res, avail;
+	int avail;
+	ssize_t res;
 	IOBuf *io = sbuf->io;
 
 	AssertActive(sbuf);
@@ -637,9 +638,9 @@ static void sbuf_try_resync(SBuf *sbuf, bool release)
 }
 
 /* actually ask kernel for more data */
-static bool sbuf_actual_recv(SBuf *sbuf, unsigned len)
+static bool sbuf_actual_recv(SBuf *sbuf, size_t len)
 {
-	int got;
+	ssize_t got;
 	IOBuf *io = sbuf->io;
 	uint8_t *dst = io->buf + io->recv_pos;
 	unsigned avail = iobuf_amount_recv(io);
@@ -818,16 +819,16 @@ failed:
 }
 
 /* send some data to listening socket */
-bool sbuf_answer(SBuf *sbuf, const void *buf, unsigned len)
+bool sbuf_answer(SBuf *sbuf, const void *buf, size_t len)
 {
-	int res;
+	ssize_t res;
 	if (sbuf->sock <= 0)
 		return false;
 	res = sbuf_op_send(sbuf, buf, len);
 	if (res < 0) {
 		log_debug("sbuf_answer: error sending: %s", strerror(errno));
 	} else if ((unsigned)res != len) {
-		log_debug("sbuf_answer: partial send: len=%d sent=%d", len, res);
+		log_debug("sbuf_answer: partial send: len=%zu sent=%zd", len, res);
 	}
 	return (unsigned)res == len;
 }
@@ -836,12 +837,12 @@ bool sbuf_answer(SBuf *sbuf, const void *buf, unsigned len)
  * Standard IO ops.
  */
 
-static int raw_sbufio_recv(struct SBuf *sbuf, void *dst, unsigned int len)
+static ssize_t raw_sbufio_recv(struct SBuf *sbuf, void *dst, size_t len)
 {
 	return safe_recv(sbuf->sock, dst, len, 0);
 }
 
-static int raw_sbufio_send(struct SBuf *sbuf, const void *data, unsigned int len)
+static ssize_t raw_sbufio_send(struct SBuf *sbuf, const void *data, size_t len)
 {
 	return safe_send(sbuf->sock, data, len, 0);
 }
@@ -1088,7 +1089,7 @@ bool sbuf_tls_connect(SBuf *sbuf, const char *hostname)
  * TLS IO ops.
  */
 
-static int tls_sbufio_recv(struct SBuf *sbuf, void *dst, unsigned int len)
+static ssize_t tls_sbufio_recv(struct SBuf *sbuf, void *dst, size_t len)
 {
 	ssize_t out = 0;
 
@@ -1098,7 +1099,7 @@ static int tls_sbufio_recv(struct SBuf *sbuf, void *dst, unsigned int len)
 	}
 
 	out = tls_read(sbuf->tls, dst, len);
-	log_noise("tls_read: req=%u out=%d", len, (int)out);
+	log_noise("tls_read: req=%zu out=%zd", len, out);
 	if (out >= 0) {
 		return out;
 	} else if (out == TLS_WANT_POLLIN) {
@@ -1113,7 +1114,7 @@ static int tls_sbufio_recv(struct SBuf *sbuf, void *dst, unsigned int len)
 	return -1;
 }
 
-static int tls_sbufio_send(struct SBuf *sbuf, const void *data, unsigned int len)
+static ssize_t tls_sbufio_send(struct SBuf *sbuf, const void *data, size_t len)
 {
 	ssize_t out;
 
@@ -1123,7 +1124,7 @@ static int tls_sbufio_send(struct SBuf *sbuf, const void *data, unsigned int len
 	}
 
 	out = tls_write(sbuf->tls, data, len);
-	log_noise("tls_write: req=%u out=%d", len, (int)out);
+	log_noise("tls_write: req=%zu out=%zd", len, out);
 	if (out >= 0) {
 		return out;
 	} else if (out == TLS_WANT_POLLOUT) {
