@@ -6,7 +6,7 @@
 cd $(dirname $0)
 
 export PGDATA=$PWD/pgdata
-export PGHOST=localhost
+export PGHOST=127.0.0.1
 export PGPORT=6667
 export EF_ALLOW_MALLOC_0=1
 export LANG=C
@@ -17,6 +17,8 @@ BOUNCER_PID=test.pid
 BOUNCER_PORT=`sed -n '/^listen_port/s/listen_port.*=[^0-9]*//p' $BOUNCER_INI`
 BOUNCER_EXE="$BOUNCER_EXE_PREFIX ../pgbouncer"
 
+BOUNCER_ADMIN_HOST=/tmp
+
 LOGDIR=log
 PG_PORT=6666
 PG_LOG=$LOGDIR/pg.log
@@ -32,6 +34,23 @@ which initdb > /dev/null || {
 	exit 1
 }
 
+# The tests require that psql can connect to the PgBouncer admin
+# console.  On platforms that have getpeereid(), this works by
+# connecting as user pgbouncer over the Unix socket.  On other
+# platforms, we have to rely on "trust" authentication, but then we
+# have to skip any tests that use authentication methods other than
+# "trust".
+case `uname` in
+	MINGW*)
+		have_getpeereid=false
+		use_unix_sockets=false
+		;;
+	*)
+		have_getpeereid=true
+		use_unix_sockets=true
+		;;
+esac
+
 # System configuration checks
 SED_ERE_OP='-E'
 case `uname` in
@@ -40,6 +59,14 @@ Linux)
 	;;
 esac
 
+case `uname` in
+MINGW*)
+	createdb() { createdb.exe "$@"; }
+	initdb() { initdb.exe "$@"; }
+	psql() { psql.exe "$@"; }
+	;;
+esac
+
 pg_majorversion=$(initdb --version | sed -n $SED_ERE_OP 's/.* ([0-9]+).*/\1/p')
 if test $pg_majorversion -ge 10; then
 	pg_supports_scram=true
@@ -47,6 +74,14 @@ else
 	pg_supports_scram=false
 fi
 
+if ! $use_unix_sockets; then
+	BOUNCER_ADMIN_HOST=127.0.0.1
+
+	cp test.ini test.ini.bak
+	sed -i 's/^unix_socket_dir =/#&/' test.ini
+	echo 'admin_users = pgbouncer' >> test.ini
+fi
+
 # System configuration checks
 if ! grep -q "^\"${USER:=$(id -un)}\"" userlist.txt; then
 	cp userlist.txt userlist.txt.bak
@@ -113,6 +148,9 @@ if [ ! -d $PGDATA ]; then
 	host   all  all  127.0.0.1/32  trust
 	host   all  all  ::1/128       trust
 	EOF
+	if ! $use_unix_sockets; then
+		sed -i 's/^local/#local/' pgdata/pg_hba.conf
+	fi
 fi
 
 pgctl start
@@ -217,6 +255,7 @@ complete() {
 	test -f $BOUNCER_PID && kill `cat $BOUNCER_PID` >/dev/null 2>&1
 	pgctl -m fast stop
 	rm -f $BOUNCER_PID
+	test -e test.ini.bak && mv test.ini.bak test.ini
 	test -e userlist.txt.bak && mv userlist.txt.bak userlist.txt
 }
 
@@ -227,18 +266,33 @@ die() {
 }
 
 admin() {
-	psql -X -h /tmp -U pgbouncer -d pgbouncer -c "$@;" || die "Cannot contact bouncer!"
+	psql -X -h $BOUNCER_ADMIN_HOST -U pgbouncer -d pgbouncer -c "$@;" || die "Cannot contact bouncer!"
 }
 
 runtest() {
 	local status
 
-	$BOUNCER_EXE -d $BOUNCER_INI
-	until psql -X -h /tmp -U pgbouncer -d pgbouncer -c "show version" 2>/dev/null 1>&2; do sleep 0.1; done
+	case `uname` in
+	MINGW*)
+		(nohup $BOUNCER_EXE $BOUNCER_INI </dev/null >/dev/null 2>&1 &)
+		;;
+	*)
+		$BOUNCER_EXE -d $BOUNCER_INI
+		;;
+	esac
+	until psql -X -h $BOUNCER_ADMIN_HOST -U pgbouncer -d pgbouncer -c "show version" 2>/dev/null 1>&2; do sleep 0.1; done
 
 	printf "`date` running $1 ... "
 	eval $1 >$LOGDIR/$1.out 2>&1
 	status=$?
+
+	# Detect fatal errors from PgBouncer (which are internal
+	# errors), but not those from PostgreSQL (which could be
+	# normal, such as authentication failures)
+	if grep 'FATAL @' $BOUNCER_LOG >> $LOGDIR/$1.out; then
+		status=1
+	fi
+
 	if [ $status -eq 0 ]; then
 		echo "ok"
 	elif [ $status -eq 77 ]; then
@@ -253,7 +307,15 @@ runtest() {
 	# allow background processing to complete
 	wait
 
-	stopit test.pid
+	case `uname` in
+	MINGW*)
+		psql -X -h $BOUNCER_ADMIN_HOST -U pgbouncer -d pgbouncer -c "shutdown;" 2>/dev/null
+		sleep 1
+		;;
+	*)
+		stopit test.pid
+		;;
+	esac
 	mv $BOUNCER_LOG $LOGDIR/$1.log
 
 	return $status
@@ -262,7 +324,7 @@ runtest() {
 # show version and --version
 test_show_version() {
 	v1=$($BOUNCER_EXE --version | head -n 1) || return 1
-	v2=$(psql -X -tAq -h /tmp -U pgbouncer -d pgbouncer -c "show version;") || return 1
+	v2=$(psql -X -tAq -h $BOUNCER_ADMIN_HOST -U pgbouncer -d pgbouncer -c "show version;") || return 1
 
 	echo "v1=$v1"
 	echo "v2=$v2"
@@ -279,10 +341,10 @@ test_show_version() {
 test_show() {
 	for what in clients config databases fds help lists pools servers sockets active_sockets stats stats_totals stats_averages users totals mem dns_hosts dns_zones; do
 		    echo "=> show $what;"
-		    psql -X -h /tmp -U pgbouncer -d pgbouncer -c "show $what;" || return 1
+		    psql -X -h $BOUNCER_ADMIN_HOST -U pgbouncer -d pgbouncer -c "show $what;" || return 1
 	done
 
-	psql -X -h /tmp -U pgbouncer -d pgbouncer -c "show bogus;" && return 1
+	psql -X -h $BOUNCER_ADMIN_HOST -U pgbouncer -d pgbouncer -c "show bogus;" && return 1
 
 	return 0
 }
@@ -394,7 +456,7 @@ test_tcp_user_timeout() {
 # server_connect_timeout
 test_server_connect_timeout_establish() {
 	psql -X -p $PG_PORT -c "alter system set pre_auth_delay to '60s'" p0
-	kill -HUP `head -n1 pgdata/postmaster.pid`
+	pgctl reload
 	sleep 1
 
 	admin "set query_timeout=3"
@@ -405,7 +467,7 @@ test_server_connect_timeout_establish() {
 	rc=$?
 
 	rm -f pgdata/postgresql.auto.conf
-	kill -HUP `head -n1 pgdata/postmaster.pid`
+	pgctl reload
 	sleep 1
 
 	return $rc
@@ -473,6 +535,10 @@ test_max_client_conn() {
 
 # - max pool size
 test_pool_size() {
+	# make existing connections go away
+	psql -X -p $PG_PORT -d postgres -c "select pg_terminate_backend(pid) from pg_stat_activity where usename='bouncer'"
+	until test $(psql -X -p $PG_PORT -d postgres -tAq -c "select count(1) from pg_stat_activity where usename='bouncer'") -eq 0; do sleep 0.1; done
+
 	docount() {
 		for i in {1..10}; do
 			psql -X -c "select pg_sleep(0.5)" $1 >/dev/null &
@@ -531,6 +597,8 @@ test_max_user_connections() {
 test_online_restart() {
 # max_client_conn=10
 # default_pool_size=5
+	$have_getpeereid || return 77
+
 	for i in {1..5}; do
 		echo "`date` attempt $i"
 
@@ -575,7 +643,7 @@ test_suspend_resume() {
 	done &
 
 	for i in {1..5}; do
-		psql -X -h /tmp -p $BOUNCER_PORT -d pgbouncer -U pgbouncer <<-PSQL_EOF
+		psql -X -h $BOUNCER_ADMIN_HOST -p $BOUNCER_PORT -d pgbouncer -U pgbouncer <<-PSQL_EOF
 		suspend;
 		\! sleep 1
 		resume;
@@ -642,7 +710,7 @@ test_database_change() {
 	sed '/^p1 =/s/dbname=p1/dbname=p0/g' test.ini >test2.ini
 	mv test2.ini test.ini
 
-	kill -HUP `cat $BOUNCER_PID`
+	admin "reload"
 
 	sleep 3
 	db2=`psql -X -tAq -c "select current_database()" p1`
@@ -692,6 +760,8 @@ test_fast_close() {
 
 # test wait_close
 test_wait_close() {
+	case `uname` in MINGW*) return 77;; esac # TODO
+
 	(
 		echo "select pg_backend_pid();"
 		sleep 3
@@ -720,6 +790,8 @@ test_wait_close() {
 
 # test auth_user
 test_auth_user() {
+	$have_getpeereid || return 77
+
 	admin "set auth_type='md5'"
 	curuser=`psql -X -d "dbname=authdb user=someuser password=anypasswd" -tAq -c "select current_user;"`
 	echo "curuser=$curuser"
@@ -762,6 +834,8 @@ test_password_server() {
 
 # test plain-text password authentication from client to PgBouncer
 test_password_client() {
+	$have_getpeereid || return 77
+
 	admin "set auth_type='plain'"
 
 	# test with users that have a plain-text password stored
@@ -809,6 +883,8 @@ test_md5_server() {
 
 # test md5 authentication from client to PgBouncer
 test_md5_client() {
+	$have_getpeereid || return 77
+
 	admin "set auth_type='md5'"
 
 	# test with users that have a plain-text password stored
@@ -851,6 +927,7 @@ test_scram_server() {
 
 # test SCRAM authentication from client to PgBouncer
 test_scram_client() {
+	$have_getpeereid || return 77
 	$pg_supports_scram || return 77
 
 	admin "set auth_type='scram-sha-256'"
@@ -891,6 +968,7 @@ test_scram_client() {
 
 # test SCRAM authentication from client to PgBouncer and on to server
 test_scram_both() {
+	$have_getpeereid || return 77
 	$pg_supports_scram || return 77
 
 	admin "set auth_type='scram-sha-256'"
@@ -909,6 +987,7 @@ test_scram_both() {
 #
 # Note: coproc requires bash >=4
 test_scram_takeover() {
+	$have_getpeereid || return 77
 	$pg_supports_scram || return 77
 
 	admin "set auth_type='scram-sha-256'"
@@ -931,6 +1010,115 @@ test_scram_takeover() {
 	test $? -eq 0
 }
 
+# Several tests that check the behavior when connecting with a
+# nonexistent user under various authentication types.  Database p1
+# has a forced user, p2 does not; these exercise slightly different
+# code paths.
+
+test_no_user_trust() {
+	admin "set auth_type='trust'"
+
+	psql -X -U nosuchuser1 -c "select 1" p2 && return 1
+	grep -F "closing because: \"trust\" authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_trust_forced_user() {
+	admin "set auth_type='trust'"
+
+	psql -X -U nosuchuser1 -c "select 1" p1 && return 1
+	grep -F "closing because: \"trust\" authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_password() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='plain'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" p2 && return 1
+	grep -F "no such user: nosuchuser1" $BOUNCER_LOG || return 1
+	grep -F "closing because: password authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_password_forced_user() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='plain'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" p1 && return 1
+	grep -F "no such user: nosuchuser1" $BOUNCER_LOG || return 1
+	grep -F "closing because: password authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_md5() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='md5'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" p2 && return 1
+	grep -F "no such user: nosuchuser1" $BOUNCER_LOG || return 1
+	grep -F "closing because: password authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_md5_forced_user() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='md5'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" p1 && return 1
+	grep -F "no such user: nosuchuser1" $BOUNCER_LOG || return 1
+	grep -F "closing because: password authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_scram() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='scram-sha-256'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" p2 && return 1
+	grep -F "no such user: nosuchuser1" $BOUNCER_LOG || return 1
+	grep -F "closing because: SASL authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_scram_forced_user() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='scram-sha-256'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" p1 && return 1
+	grep -F "no such user: nosuchuser1" $BOUNCER_LOG || return 1
+	grep -F "closing because: SASL authentication failed" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
+test_no_user_auth_user() {
+	$have_getpeereid || return 77
+
+	admin "set auth_type='md5'"
+
+	PGPASSWORD=whatever psql -X -U nosuchuser1 -c "select 1" authdb && return 1
+	# Currently no mock authentication when using
+	# auth_query/auth_user.  See TODO in
+	# handle_auth_query_response().
+	grep -F "closing because: no such user (age" $BOUNCER_LOG || return 1
+
+	return 0
+}
+
 testlist="
 test_show_version
 test_show
@@ -966,6 +1154,15 @@ test_scram_server
 test_scram_client
 test_scram_both
 test_scram_takeover
+test_no_user_trust
+test_no_user_trust_forced_user
+test_no_user_password
+test_no_user_password_forced_user
+test_no_user_md5
+test_no_user_md5_forced_user
+test_no_user_scram
+test_no_user_scram_forced_user
+test_no_user_auth_user
 "
 
 if [ $# -gt 0 ]; then
